import sys
import os
from .GOESLayer import GOESLayer
from .frametypes import MPDUMsg, CPPDUMsg
from .opsmsgs import CPPDUOpsMsg
from .CRC import CRC
import logging
import multiprocessing

class CPPDULayer(GOESLayer):

    class MPDUHandlerStateMachine(object):

        class TransportQueueStateMachine(object):

            def __init__(self):
                self.logger = logging.getLogger(__name__)
                self.dataBuffer = []
                self.output = CPPDUMsg()
                self.outOpsMsg = CPPDUOpsMsg()
                self.transportQFunc = self.__processQWait
                self.firstHeaderPointer = 0
                self.needMoreData = True
                self.hasProduct = False
                #Let state machine decide whether it needs more data

            def __processQWait(self):
                self.hasProduct = False
                #print("First Header Pointer Aligned: {0}".format(self.firstHeaderPointer))
                #print("Considered First Header Ptr: {0}".format(self.firstHeaderPointer))
                if len(self.dataBuffer) >= CPPDUMsg.HEADER_SIZE_BYTES:
                    self.output = CPPDUMsg()
                    self.outOpsMsg = CPPDUOpsMsg()
                    self.output.getHeader(self.dataBuffer[0:CPPDUMsg.HEADER_SIZE_BYTES])
                    self.dataBuffer = self.dataBuffer[CPPDUMsg.HEADER_SIZE_BYTES:len(self.dataBuffer)]
                    self.bytesLeft = self.output.packetLength
                    self.transportQFunc = self.__processQBuild
                    self.needMoreData = False
                else:
                    self.needMoreData = True
                #print("Leftover data Header: {0}".format(len(self.dataBuffer)))
            def __processQBuild(self):
                #print("Begin Buffer Length: {0}".format(len(self.dataBuffer)))
                #print("Begin Bytes LEft: {0}".format(self.bytesLeft))
                #print("First Header Pointer As of Begin: {0}".format(self.firstHeaderPointer))
                #output = False means the processQ function should stop and get new input
                #not that the function failed in any way

                if self.firstHeaderPointer < MPDUMsg.NOT_NEW_FRAME and \
                    self.firstHeaderPointer < self.bytesLeft < self.output.packetLength:
                    self.flush()
                    self.logger.warn("New Transport Frame came early! Flushing!")
                else:
                    if (self.bytesLeft > len(self.dataBuffer)):
                        self.output.fromFrame(self.dataBuffer)
                        self.bytesLeft -= len(self.dataBuffer)
                        #print("Bytes Left after Full Buffer Consume: {0}".format(self.bytesLeft))
                        self.dataBuffer = []
                        self.needMoreData = True
                        self.hasProduct = False
                        #print("Leftover data Plenty: {0}".format(len(self.dataBuffer)))
                    else:
                        #Else one frame ends and another begins
                        self.output.fromFrame(self.dataBuffer[0:self.bytesLeft])
                        #print("Supposed Buffer Length: {0}".format(len(self.dataBuffer)-self.bytesLeft))
                        self.dataBuffer = self.dataBuffer[self.bytesLeft:len(self.dataBuffer)]
                        #print("Actual End Buffer Length: {0}".format(len(self.dataBuffer)))
                        self.bytesLeft = 0
                        #Prevent fill packets from being passed forward
                        #print("Leftover data Partial: {0}".format(len(self.dataBuffer)))
                        if (self.output.apid != CPPDUMsg.APID_FILL_PACKET):
                            self.output.getCrcFromData()
                            self.hasProduct = True
                            self.outOpsMsg.setHasCrcErrors(not CRC.Calculate(self.output.data, self.output.crc))
                        else:
                            self.logger.info('Fill Packet detected')

                        self.needMoreData = False
                        #TODO: Handle CRC
                        self.transportQFunc = self.__processQWait

            def processQ(self):
                while not self.needMoreData and not self.hasProduct:
                    self.transportQFunc()
                if self.hasProduct:
                    return (self.output, self.outOpsMsg)
                else:
                    return (None, CPPDUOpsMsg())

            #Concatenate existing data and new data, essentially acting like a queue
            def enqueue(self, data, opsMsg):
                self.dataBuffer += data.data
                self.firstHeaderPointer = data.firstHeaderPointer
              # print("First Header Pointer Raw: {0}".format(data.firstHeaderPointer))
                #print("Enqueued Data Length: {0}".format(len(self.dataBuffer)))
                self.needMoreData = False
                self.hasProduct = False

            def flush(self):
                dataFlushed = len(self.output.data) + len(self.dataBuffer)
                # If we're flushing data, only flush data before the FHP
                #TODO: What if FHP is 2047???
                self.dataBuffer = self.dataBuffer[len(self.dataBuffer) - 884 + self.firstHeaderPointer:len(self.dataBuffer)]
                #self.dataBuffer = self.dataBuffer[self.firstHeaderPointer:len(self.dataBuffer)]
                self.output = CPPDUMsg()
                self.outOpsMsg = CPPDUOpsMsg()
                self.transportQFunc = self.__processQWait
                self.firstHeaderPointer = 0
                self.needMoreData = True
                self.hasProduct = False
                return dataFlushed

            def HasProduct(self):
                return self.hasProduct


        def __init__(self):
            self.logger = logging.getLogger(__name__)
            self.mpduHandlerFunc = self.__processDataFlush
            self.transportQueue = self.TransportQueueStateMachine()
            self.flushedBytesCount = 0

        #Flush data unless it contains a header for a new transport packet
        def __processDataFlush(self, data, opsMsg):
            outOpsMsg = CPPDUOpsMsg()
            output = []
            self.flushedBytesCount += len(data.data)
            #Detected new Transport frame in MPDU
            if opsMsg.getIsNewTransportFrame():
                #print("New Transport First Header Ptr: {0}".format(data.firstHeaderPointer))
                output,outOpsMsg = self.__enterEnqueueState(data, opsMsg)
            return (output, outOpsMsg)

        #Add data to Transport Queue and process it
        def __processDataEnqueue(self, data, opsMsg):
            if self.flushedBytesCount > 0:
                self.logger.warn('Flushed %d bytes', self.flushedBytesCount)
                self.flushedBytesCount = 0
            outOpsMsg = CPPDUOpsMsg()
            output = []
            if opsMsg.hasMissedFrames():
                self.logger.warn("Missed Frames!  Flushing Data")
                self.__enterFlushState()
            else:
                self.transportQueue.enqueue(data, opsMsg)
                (output, outOpsMsg) = self.transportQueue.processQ()
            return (output, outOpsMsg)

        def HasProduct(self):
            return self.transportQueue.HasProduct()


        def __enterEnqueueState(self, data, opsMsg):
            self.mpduHandlerFunc = self.__processDataEnqueue
            #Throw away junk bytes from before "First valid transport frame"
            #print("ENTERING ENQUEUE STATE")
            data.data = data.data[data.firstHeaderPointer:len(data.data)]
            return self.mpduHandlerFunc(data, opsMsg)

        def __enterFlushState(self):
            self.mpduHandlerFunc = self.__processDataFlush
            self.flushedBytesCount = self.transportQueue.flush()


        def __createTransportFrame(self):
            self.output = CPPDUMsg()
            self.outOpsMsg = CPPDUOpsMsg()

    def __init__(self):
        GOESLayer.__init__(self)
        self.logger = logging.getLogger(__name__)
        self.mpduHandler = self.MPDUHandlerStateMachine()
        self.logger.setLevel(logging.DEBUG)

    def processData(self, data, opsMsg):
#        print(data)
        self.logger.debug("Incoming MPDU - VCID=%d %s", opsMsg.vcID, ' '.join(str(data).split('\n')))
        return self.mpduHandler.mpduHandlerFunc(data, opsMsg) + (self.mpduHandler.transportQueue.needMoreData,)
