from AbstractHeader import AbstractHeader

##
# @brief Image Data header class.
# @details Contains an encoded text description of the contents of the image
# For example, engineering units, mappings of overlays to other images, etc
# @author Miguel Dominguez
# @date 2016
class ImageDataHeader(AbstractHeader):

   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 3
   @staticmethod
   def headerIndex():
      return 3

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.data = "".join(map(chr,data[3:self.headerRecordLength]))
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Image Data Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Data: {0}\n".format(self.data)
      return output
