from AbstractHeader import AbstractHeader

##
# @brief Rice Compression header class.
# @details Identifies Rice compression parameters for compressed
# files
# @author Miguel Dominguez
# @date 2016
class RiceCompressionHeader(AbstractHeader):

   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 131
   @staticmethod
   def headerIndex():
      return 131

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.flags = (data[3] << self.BYTE_SHIFT) | data[4]
      mask = 0x0001
      shift = 0
      self.allowK13 = bool(self.flags & mask)
      mask <<= 1
      shift += 1
      self.chip = bool((self.flags & mask) >> shift)
      mask <<= 1
      shift += 1
      self.ec = bool((self.flags & mask) >> shift)
      mask <<= 1
      shift += 1
      self.lsb = bool((self.flags & mask) >> shift)
      mask <<= 1
      shift += 1
      self.msb = bool((self.flags & mask) >> shift)
      mask <<= 1
      shift += 1
      self.nn = bool((self.flags & mask) >> shift)
      mask <<= 2
      shift += 2
      self.raw = bool((self.flags & mask) >> shift)
      self.pixelsPerBlock = data[5]
      self.scanLinesPerPacket = data[6]
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Rice Compression Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Allow K13: {0}\n".format(self.allowK13)
      output += "Chip: {0}\n".format(self.chip)
      output += "EC: {0}\n".format(self.ec)
      output += "LSB: {0}\n".format(self.lsb)
      output += "MSB: {0}\n".format(self.msb)
      output += "NN: {0}\n".format(self.nn)
      output += "Raw: {0}\n".format(self.raw)
      output += "Pixels Per Block: {0}\n".format(self.pixelsPerBlock)
      output += "Scan Lines Per Packet: {0}\n".format(self.scanLinesPerPacket)
      return output
